/*
 * 
 * Przykad wspbienoci: dwie sesje pobieraj ten sam rekord, modyfikuj i chc go zapisa sekwencyjnie.
 * 
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;

using DevExpress.Xpo;
using DevExpress.Xpo.DB;

namespace ConcurrencyExample
{
    class Person : XPObject
    {
        public Person() { }
        public Person(Session Session) : base(Session) { }

        private string firstName;
        private string lastName;
        private string email;

        [Size(30)]
        public string FirstName
        {
            get { return firstName; }
            set { firstName = value; }
        }
        [Size(50)]
        public string LastName
        {
            get { return lastName; }
            set { lastName = value; }
        }
        [Size(100)]
        public string Email
        {
            get { return email; }
            set { email = value; }
        }
    }
    class Program
    {
        static void Main(string[] args)
        {
            string connectionString = ConfigurationManager.AppSettings["ConnectionString"];

            XpoDefault.DataLayer = XpoDefault.GetDataLayer(connectionString, AutoCreateOption.DatabaseAndSchema);
            XpoDefault.Session.ClearDatabase();

            Person person = new Person();
            person.FirstName = "Jan";
            person.LastName = "Kowalski";
            person.Email = "jan.kowalski@efirma.com";
            person.Save();

            Session session1 = new Session();
            session1.AutoCreateOption = AutoCreateOption.DatabaseAndSchema;
            session1.LockingOption = LockingOption.Optimistic;
            //session1.OptimisticLockingReadBehavior = OptimisticLockingReadBehavior.ThrowException;
            session1.ConnectionString = connectionString;

            Person p1 = (Person)(new XPCollection(session1, typeof(Person)))[0];

            Session session2 = new Session();
            session2.AutoCreateOption = AutoCreateOption.DatabaseAndSchema;
            session2.LockingOption = LockingOption.None;
            session2.OptimisticLockingReadBehavior = OptimisticLockingReadBehavior.ThrowException;
            session2.ConnectionString = connectionString;

            XPCollection<Person> persons = new XPCollection<Person>(session2);
            Person p2 = persons[0];

            Console.WriteLine("{0} {1}", p1.FirstName, p1.LastName);
            Console.WriteLine("{0} {1}", p2.FirstName, p2.LastName);

            p1.FirstName = "Janek";
            p1.Save();

            // To czy zadziaa ponisze ma wpyw ustawienie LockingOption
            p2.FirstName = "Jancio";
            p2.Save();

            // Na ponisze dwie linijki ma wpyw ustawienie OptimisticLockingReadBehavior
            persons.Reload();
            Person p3 = (Person)persons[0];

            Console.WriteLine("{0} {1}", p1.FirstName, p1.LastName);
            Console.WriteLine("{0} {1}", p2.FirstName, p2.LastName);

            Console.ReadLine();
        }
    }
}
