/*
 * 
 * Przykad wykonywania zapyta wraz z sortowaniem
 * 
 */

using System;
using System.Collections.Generic;
using System.Text;

using DevExpress.Xpo;
using System.Configuration;
using DevExpress.Data.Filtering;
using DevExpress.Xpo.DB;

namespace QueryingDatastore2
{
    class HTTPSession : XPObject
    {
        private string userName;

        public string UserName
        {
            get { return userName; }
            set { userName = value; }
        }
        private DateTime timestamp;

        public DateTime Timestamp
        {
            get { return timestamp; }
            set { timestamp = value; }
        }

        public HTTPSession() { }
        public HTTPSession( Session Session ) : base( Session ) { }
    }

    /* ----------------------------------- */

    class Oddzial : XPObject
    {
        public string Nazwa;
        public int Kod;
        public Oddzial( string Nazwa, int Kod  )
        {
            this.Nazwa = Nazwa;
            this.Kod = Kod;
        }
        public Oddzial( Session Session ) : base( Session ) { }
        [Association( "OddzialPracownik" )]
        public XPCollection<Pracownik> Pracownicy { get { return GetCollection<Pracownik>( "Pracownicy" ); } }
    }
    class Pracownik : XPObject
    {
        public string Imie;
        public string Nazwisko;
        public string Ulica;
        public string Miasto;
        public string KodPocztowy;
        public int RokUrodzenia;
        public Pracownik( Oddzial Oddzial, string Imie, string Nazwisko, string Ulica, string Miasto,
            string KodPocztowy, int RokUrodzenia )
        {
            this.Oddzial = Oddzial; this.Imie = Imie; this.Nazwisko = Nazwisko; this.Ulica = Ulica;
            this.Miasto = Miasto; this.KodPocztowy = KodPocztowy; this.RokUrodzenia = RokUrodzenia;
        }
        public Pracownik( Session Session ) : base( Session ) { }
        [Association( "OddzialPracownik" )]
        public Oddzial Oddzial;
    }


    class Program
    {
        static void Main( string[] args )
        {
            XpoDefault.ConnectionString = ConfigurationManager.AppSettings["ConnectionString"];
            XpoDefault.Session.ClearDatabase();

            Oddzial o0 = new Oddzial( "Kozie Z.", 1000 );
            o0.Save();
            Oddzial o1 = new Oddzial( "Krakw", 1001 );
            new Pracownik( o1, "Jan01", "Kowalski01", "ul. Stroma 22", "Wrocaw", "51-111", 1964 ).Save();
            new Pracownik( o1, "Jan02", "Kowalski02", "ul. Stroma 22", "Wrocaw", "51-111", 1965 ).Save();
            new Pracownik( o1, "Jan03", "Kowalski03", "ul. Stroma 22", "Wrocaw", "51-111", 1966 ).Save();
            new Pracownik( o1, "Ewa01", "Nowak01", "ul. Zawiszy 43", "Wrocaw", "51-112", 1965 ).Save();
            Oddzial o2 = new Oddzial( "Wrocaw", 1002 );
            new Pracownik( o2, "Ewa02", "Nowak02", "ul. Zawiszy 43", "Wrocaw", "51-112", 1966 ).Save();
            new Pracownik( o2, "Ewa03", "Nowak03", "ul. Zawiszy 43", "Wrocaw", "51-112", 1967 ).Save();
            Oddzial o3 = new Oddzial( "Warszawa", 1003 );
            new Pracownik( o3, "Marcin01", "Maliniak01", "ul. Truskawkowa 13", "Warszawa", "00-411", 1958 ).Save();
            new Pracownik( o3, "Marcin02", "Maliniak02", "ul. Truskawkowa 13", "Warszawa", "00-411", 1960 ).Save();
            Oddzial o4 = new Oddzial( "Pozna", 1004 );
            new Pracownik( o4, "Marcin03", "Maliniak03", "ul. Truskawkowa 13", "Warszawa", "00-411", 1964 ).Save();

            Console.WriteLine( "-------- Proste zapytanie z sortowaniem" );
            XPCollection<Pracownik> collection = new XPCollection<Pracownik>(
                //CriteriaOperator.Parse( "Nazwisko LIKE 'Kowalski%' AND Imie LIKE '%a%'" ),
                new BinaryOperator( "RokUrodzenia", 1964, BinaryOperatorType.GreaterOrEqual ),
                new SortProperty[] { 
                    new SortProperty( "Nazwisko", SortingDirection.Ascending ),
                    new SortProperty( "Imie", SortingDirection.Ascending )
                }
            );
            foreach ( Pracownik o in collection )
            {
                Console.WriteLine( "{0} {1}, {2}, {3} {4}", o.Imie, o.Nazwisko, o.Ulica, o.KodPocztowy, o.Miasto );
            }

            Console.WriteLine( "-------- Oddzialy bez pracownikow" );
            XPCollection<Oddzial> query5 = new XPCollection<Oddzial>();
            query5.Criteria = CriteriaOperator.Parse( "Pracownicy.Count=0" );
            foreach ( Oddzial d in query5 )
            {
                Console.WriteLine( "{0}", d.Nazwa );
            }


            Console.WriteLine( "-------- Oddziay posiadajce co najmniej dwch pracownikw" );
            Console.WriteLine( "         urodzonych midzy 1965 i 1980" );
            XPCollection<Oddzial> collection2 = new XPCollection<Oddzial>(
                CriteriaOperator.Parse( "Pracownicy[RokUrodzenia Between(1965, 1980) AND Nazwisko LIKE '%'].Count>=2" ), null );
            foreach ( Oddzial o in collection2 )
            {
                Console.WriteLine( o.Nazwa );
            }

            Console.WriteLine( "-------- Pracownicy, ktrzy s z oddziau Wrocaw" );
            XPCollection<Pracownik> query2 = new XPCollection<Pracownik>();
            query2.Criteria = new InOperator( "Oddzial", new XPCollection<Oddzial>( new BinaryOperator( "Nazwa", "Wrocaw" ) ) );

            foreach ( Pracownik p in query2 )
            {
                Console.WriteLine( "{0} {1}", p.Imie, p.Nazwisko );
            }

            Console.WriteLine( "-------- Pracownicy urodzeni po 1966 (inna skadnia)" );
            XPCollection<Pracownik> query10 = new XPCollection<Pracownik>();
            query10.Criteria = GroupOperator.And(
                new OperandProperty( "RokUrodzenia" ) > new OperandValue( 1966 ),
                new OperandProperty( "Oddzial.Nazwa" ) == new OperandValue( "Wrocaw" )
            );

            foreach ( Pracownik p in query10 )
            {
                Console.WriteLine( "{0} {1}, {2}", p.Imie, p.Nazwisko, p.RokUrodzenia );
            }



            HTTPSession session1 = new HTTPSession( XpoDefault.Session );
            session1.UserName = "ewa";
            session1.Timestamp = new DateTime( 2004, 11, 4 );
            session1.Save();

            HTTPSession session2 = new HTTPSession( XpoDefault.Session );
            session2.UserName = "pawel";
            session2.Timestamp = new DateTime( 2003, 12, 10 );
            session2.Save();

            HTTPSession session3 = new HTTPSession( XpoDefault.Session );
            session3.UserName = "pawel";
            session3.Timestamp = new DateTime( 2001, 10, 22 );
            session3.Save();

            HTTPSession session4 = new HTTPSession( XpoDefault.Session );
            session4.UserName = "pawel";
            session4.Timestamp = new DateTime( 2002, 7, 15 );
            session4.Save();

            Console.WriteLine( "-------- Najnowsza sesja" );

            Session s = new Session();
            s.ConnectionString = XpoDefault.Session.ConnectionString;


            // Zapytanie: select top 1 username, timestamp from sessions where username=pawel order by timestamp desc
            // Dla tabeli Sessions( UserName, Timestamp )
            XPCollection<HTTPSession> sessions = new XPCollection<HTTPSession>( s );
            sessions.Criteria = CriteriaOperator.Parse( "UserName == 'pawel'" );
            sessions.Sorting.Add( new SortProperty( "Timestamp", SortingDirection.Descending ) );
            sessions.TopReturnedObjects = 1;
            foreach ( HTTPSession ss in sessions )
            {
                Console.WriteLine( ss.UserName + " " + ss.Timestamp );
            }

            Console.ReadLine();

        }
    }
}
